package eu.nets.sis.idrights.demoapp.controller;

import static org.junit.Assert.assertTrue;
import static org.mockito.BDDMockito.given;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.ResultMatcher;

import eu.nets.sis.idrights.demoapp.service.IDRightsService;

@RunWith(SpringRunner.class)
@WebMvcTest(ApplicationController.class)
public class ApplicationControllerTest {
	
	@Autowired
    private MockMvc mvc;
	
	@MockBean
	IDRightsService idrightsService;

	@Test
	public void testGetSPStatus() throws Exception {
		
		given(this.idrightsService.getSPInfoByOrgXmlRequest("NO", "923609016", null))
        .willReturn(null);
		
		this.mvc.perform(get("/getsignaturedatastatus").accept("text/plain").param("countrycode", "NO").param("organizationNo", "923609016"))
        .andExpect(status().isOk()).andExpect(redirectedPage("error")); // Error page
		
	}
	
	@Test
	public void testVerifySPStatus() throws Exception {
		
		given(this.idrightsService.verifySPInfoXmlRequest("NO", "923609016", "08025617909"))
        .willReturn(null);
		
		this.mvc.perform(get("/verifysignaturedatastatus").accept("text/plain").param("countrycode", "NO").param("organizationNo", "923609016").param("ssn", "08025617909"))
        .andExpect(status().isOk()).andExpect(redirectedPage("error")); // Error page
		
	}
	
	// match view page (aka Location page)
    private static ResultMatcher redirectedPage(String viewPage) {
        return new ResultMatcher() {
        	@Override
            public void match(MvcResult result) {
                assertTrue(result.getModelAndView().getViewName().equals(viewPage));
            }
        };
    }
}
