package eu.nets.sis.idrights.demoapp.util;

import static org.junit.Assert.*;

import java.util.Date;


import org.junit.Test;

import no.bbs.trust.b2b.xmlapi.message.domain.B2BMessage;
import no.bbs.trust.b2b.xmlapi.message.generated.TrustB2BMessage;
import no.bbs.trust.b2b.xmlapi.message.generated.TrustB2BStdHeaderType;

public class IDRightsUtilTest {
	@Test
	public void testConvertB2BMsgToString() {
		B2BMessage b2bMessage = createB2BMessage("123456789", 1001);
		String xml = IDRightsUtil.convertB2BMsgToString(b2bMessage);
		assertNotNull(xml);

	}
	
	@Test
	public void testConvertToXMLStringFormat() {
		String xml = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" +
		"<TrustB2BMessage xmlns=\"http://www.nets.eu/esec/trustb2b/2011/10/tb2b#\">" +
		"<TrustB2BHeader><MerchantID>1001</MerchantID><Time>2017-10-18T10:10:03.666+05:30</Time><MessageID>1001-1508301603666</MessageID><CountryCode>NO</CountryCode><TraceID>123456789</TraceID></TrustB2BHeader></TrustB2BMessage>";
		String result = IDRightsUtil.convertToXMLStringFormat(xml);
		assertNotEquals(xml, result);
	}
	
	private static B2BMessage createB2BMessage(String transactionId, long merchantId) {
		TrustB2BMessage trustB2bMessage = new TrustB2BMessage();
		trustB2bMessage.setTrustB2BHeader(createHeader(transactionId, merchantId));
		B2BMessage b2bMessage = new B2BMessage(trustB2bMessage, transactionId);
		b2bMessage.getTrustB2BMessage().getTrustB2BHeader().setTraceID(transactionId);
		
		return b2bMessage;
	}

	private static TrustB2BStdHeaderType createHeader(String transactionId, long merchantId) {
		TrustB2BStdHeaderType trustB2bHeader = new TrustB2BStdHeaderType();
		trustB2bHeader.setCountryCode("NO");
		trustB2bHeader.setMerchantID(merchantId);
		trustB2bHeader.setMessageID(merchantId + "-" + System.currentTimeMillis()); // MessageID should be sent by Merchant always, and it should always be unique
		trustB2bHeader.setTraceID(transactionId);
		trustB2bHeader.setTime(new Date());
		return trustB2bHeader;
	}
}
